import java.util.*;
import javax.vecmath.*;


class Hole {
private int id;
private Egde e;
private int length;
private boolean filled = false;
private ArrayList edges = new ArrayList();
private ArrayList newEdges = new ArrayList();
private ArrayList newTrekanter = new ArrayList();
private double eps = 0.14; //error treshold in radians

	public Hole(int id, Egde e){
		this.id=id;
		this.e=e;

	}

	public String toString(){
		return id + " " + length + "\n";
	}

	public Egde getEgde(){
		return e;
	}

	public void printNewEdges(){
		System.out.println(newEdges.size());
		for(int i=0;i<newEdges.size();i++){
			System.out.println((Egde) newEdges.get(i));
		}
		System.out.println(newTrekanter.size());
		for(int i=0;i<newTrekanter.size();i++){
			System.out.println((Trekant) newTrekanter.get(i));
		}
	}

	public ArrayList getNewEdges(){
		return newEdges;
	}

	public ArrayList getNewTrekanter(){
		return newTrekanter;
	}

	public boolean getFilled(){
		return filled;
	}

	public void setFilled(){
		filled = true;
	}

	public int getId(){
		return id;
	}

	public int getLength(){
		return length;
	}

	public  void setLength(int length){
		this.length = length;
	}

	public void addEdge(Egde e){
		edges.add(e);
	}

	public ArrayList getEdges(){
		return edges;
	}

	//henter neste kant ArrayList edges
	public Egde getNextEdge(int i){
		if(i<edges.size()-1){
			return (Egde) edges.get(i+1);
		}
		else if(i==edges.size()-1){
			return (Egde) edges.get(0);
		}
		else if (i==edges.size()){
			return (Egde) edges.get(1);
		}
		else{
			System.out.println("Invalid input index in getNextEdge");
			return null;
		}
	}

	//henter forrige kant i ArrayList edges
	public Egde getPreviousEdge(int i){
		if(i>1){
			return (Egde) edges.get(i-1);
		}
		else if(i==0){
			return (Egde) edges.get(edges.size()-1);
		}
		else{
			System.out.println("Invalid input index in getPreviousEdge");
			return null;
		}
	}

	//algoritme for 3D hullfylling
	public void fill3DHole(){

		if(edges.size()>3){
			//tar de to frste kantene i edges
			Egde e_first = (Egde) edges.get(0);
			Egde e_second = (Egde) edges.get(1);

			Node n1 = e_first.getSource();
			Node n2 = e_second.getSource();
			Node n3 = getNextEdge(1).getSource();

			Vector3f v1 = new Vector3f((float)n1.getX() - (float)n2.getX(), (float)n1.getY() - (float)n2.getY(), (float)n1.getZ() - (float)n2.getZ());
			Vector3f v2 = new Vector3f((float)n3.getX() - (float)n2.getX(), (float)n3.getY() - (float)n2.getY(), (float)n3.getZ() - (float)n2.getZ());

			//normalen til planet definert av de to frste kantene
			Vector3f n = new Vector3f();
			n.cross(v1,v2);

			int next = 1;
			int pre = 0;
			ArrayList hole2D = new ArrayList();
			//legger til de to kantene til 2D hull lista
			hole2D.add(e_first);
			hole2D.add(e_second);

			double next_angle = 0;
			double pre_angle = 0;

			//leter FREMOVER i edges etter kanter som ligger i planet definert av de to frste kantene
			//terminerer nr en kant er funnet som ikke ligger i dette planet
			while(Math.abs(next_angle)<eps && next<edges.size()-1){

				Egde e_next = getNextEdge(next);
				Egde e_this = (Egde) edges.get(next);

				Node n_this = e_this.getSource();
				Node n_next = e_next.getSource();

				Vector3f v = new Vector3f((float) n_next.getX() - (float)n_this.getX(), (float) n_next.getY() - (float)n_this.getY(),(float) n_next.getZ() - (float)n_this.getZ());

				next_angle = Math.abs(n.dot(v));

				if(next_angle<eps){
					hole2D.add(e_next);

				}
				next++;

			}// slutt while

			//leter BAKOVER i edges etter kanter som ligger i planet definert av de to frste kantene
			//terminerer nr en kant er funnet som ikke ligger i dette planet
			while(pre_angle<eps && hole2D.size()<edges.size()){
				Egde e_pre = getPreviousEdge(pre);
				Egde e_this = (Egde) edges.get(pre);

				Node n_this = e_this.getSource();
				Node n_pre = e_pre.getSource();

				Vector3f v = new Vector3f((float) n_pre.getX() - (float)n_this.getX(), (float) n_pre.getY() - (float)n_this.getY(),(float) n_pre.getZ() - (float)n_this.getZ());

				pre_angle = Math.abs(n.dot(v));

				if(pre_angle<eps){
					//legger til frst i 2D hull lista
					hole2D.add(0,e_pre);
					if(pre==0){
						pre=edges.size();
					}
					pre--;
				}
			}//slutt while

			//oppdaterer edges og 2D hull lista og legger til "bridge" kanten
			if(hole2D.size()<edges.size()){
				Egde siste = (Egde) hole2D.get(hole2D.size()-1);
				Egde forste = (Egde) hole2D.get(0);
				Node e1s = getNextEdge(next-1).getSource();
				Node e2s = forste.getSource();

				if(!e1s.equals(e2s)){
					Egde e1 = new Egde(e1s,0);
					Egde e2 = new Egde(e2s,0);

					e1.setTwin(e2);
					e2.setTwin(e1);

					newEdges.add(e1);
					newEdges.add(e2);
					hole2D.add(e1);

					while(next-1>=0){
						edges.remove(0);
						next--;
					}

					while(pre<edges.size() && pre>0){
						edges.remove(edges.size()-1);
						pre++;
					}
					edges.add(0,e2);
				}
				else{
					while(next-1>=0){
						edges.remove(0);
						next--;
					}
					while(pre<edges.size() && pre>0){
						edges.remove(edges.size()-1);
						pre++;
					}
				}
			}
			else{
				while(next-1>=0){
					edges.remove(0);
					next--;
				}
				while(pre<edges.size() && pre>0){
					edges.remove(edges.size()-1);
					pre++;
				}

			}

			fillhole2D(hole2D);
			fill3DHole();
		} //slutt if
		else if(edges.size()==3){
			edges = fillTriangle(edges);
		}//slutt elseif
	}// slutt metode


	//protruding point algoritmen, fyller et 2D hull
	public void fillhole2D(ArrayList hole2D){
		if(hole2D.size()>3){
			double angle = Math.PI;
			int i = 0;
			Egde e1= null;
			Egde e2= null;
			Node n3 = null;

			//finner to kanter med vinkel mindre enn pi mellom seg
			while(angle>=Math.PI-0.14){
				e1 = (Egde) hole2D.get(i);
				Node n1 = e1.getSource();
				if(i==hole2D.size()-1){
					i=-1;
				}//slutt if
				e2 = (Egde) hole2D.get(i+1);
				Node n2 = e2.getSource();
				if(i==hole2D.size()-2){
					i = -2;
				}
				Egde e3 = (Egde) hole2D.get(i+2);
				n3 = e3.getSource();

				angle = M.findAngle(n1,n2,n3);
				i++;
			} // slutt while

			//legger til ny kant mellom de to kantene man fant
			//oppdaterer pekere og hull lister
			Egde e3twin = new Egde(e1.getSource(),0);
			Egde e1twin = e1.getTwin();
			Egde e2twin = e2.getTwin();

			if(e1twin==null){
				e1twin = new Egde(e2.getSource(),0);
				newEdges.add(e1twin);
			}
			if(e2twin==null){
				e2twin = new Egde(n3,0);
				newEdges.add(e2twin);
			}

			e1.setTwin(e1twin);
			e1twin.setTwin(e1);
			e2.setTwin(e2twin);
			e2twin.setTwin(e2);

			e1twin.setNext(e2twin);
			e2twin.setNext(e3twin);
			e3twin.setNext(e1twin);

			Trekant t = new Trekant(n3, e2.getSource(), e1.getSource(),0);
			t.setEgde(e1twin);

			e1twin.setTrekant(t);
			e2twin.setTrekant(t);
			e3twin.setTrekant(t);

			newEdges.add(e3twin);
			newTrekanter.add(t);

			if(i==-1){
				i = hole2D.size()-1;
			}
			hole2D.remove(i);
			if(i==0){
				i = hole2D.size()-1;
			}
			hole2D.remove(i-1);
			hole2D.add(i-1, e3twin);

			fillhole2D(hole2D);
		} //slutt if
		else if (hole2D.size()==3){
			hole2D = fillTriangle(hole2D);
		} //slutt elseif
	} //slutt metode


	//tar en boudnary loop bestende av 3 kanter og fyller inn en trekant
	public ArrayList fillTriangle(ArrayList triangle){
		Egde e1 = (Egde) triangle.get(0);
		Egde e2 = (Egde) triangle.get(1);
		Egde e3 = (Egde) triangle.get(2);

			Egde e1twin = e1.getTwin();
			Egde e2twin = e2.getTwin();
			Egde e3twin = e3.getTwin();

			if(e1twin==null){
				e1twin = new Egde(e2.getSource(),0);
				newEdges.add(e1twin);
			}
			if(e2twin==null){
				e2twin = new Egde(e3.getSource(),0);
				newEdges.add(e2twin);
			}
			if(e3twin==null){
				e3twin = new Egde(e1.getSource(),0);
				newEdges.add(e3twin);
			}
			e1twin.setNext(e2twin);
			e2twin.setNext(e3twin);
			e3twin.setNext(e1twin);

			Trekant t = new Trekant(e3.getSource(), e2.getSource(), e1.getSource(),0);
			t.setEgde(e1twin);

			e1twin.setTrekant(t);
			e2twin.setTrekant(t);
			e3twin.setTrekant(t);

			e3.setTwin(e3twin);
			e3twin.setTwin(e3);
			e1.setTwin(e1twin);
			e1twin.setTwin(e1);
			e2.setTwin(e2twin);
			e2twin.setTwin(e2);

			newTrekanter.add(t);

		triangle.remove(2);
		triangle.remove(1);
		triangle.remove(0);

		return triangle;
	}//slutt metode
} //slutt klasse
